from .chart import Chart
import matplotlib.pyplot as plt

class StackedAreaChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the StackedAreaChart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = False
        check_var = self._is_var_exist(self._numerical_column, 1) and self._is_var_exist(self._date_column, 1)
        if check_var:
            if (len(self._numerical_column) == 1):
                if len(self.dataframe[self._date_column[0]].unique()) == len(self.dataframe):
                    is_promote = True
                else:
                    is_promote = False
            else:
                is_promote = True

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating StackedAreaChart visualization

        Returns:
            (string) date_label: date label  for axis-x
            (list) numerical_columns: numerical list
        """
        date_label = None
        numerical_column = None

        if self._is_var_exist(self._date_column, 1):
            date_label = self._date_column[0]
            if self._is_var_exist(self._numerical_column, 1):
                numerical_column = self._numerical_column
        
        return date_label, numerical_column          
  

    def draw(self):
        """
        Generate StackedAreaChart visualization
        """
        date_label, numerical_column  = self._check_requirements()

        if date_label is not None and numerical_column is not None:
            #set index by date label
            dataframe = self.dataframe.copy()
            dataframe = dataframe.set_index(date_label)
            #plot
            self.figsize = self.__set_figsize(self.kwargs.get('figsize'))
            #check if param figsize exist
            if self.figsize is not None:
                ax = dataframe.plot.area(stacked=True, figsize=self.figsize)
                plt.show(block=True)
            else:
                ax = dataframe.plot.area(stacked=True, figsize=(15,10))
                plt.show(block=True)

    @staticmethod
    def __set_figsize(figsize_input):
        """
        Setter of figsize based on figsize input for matplotlib chart

        Parameters:
            (tuple) figsize_input: The figsize input

        Returns:
            (tuple) figsize: The result figsize  
        """
        figsize = None
        is_numeric_value = None

        try:
            if figsize_input is not None and len(figsize_input) == 2:
                is_numeric_value = all(isinstance(v, int) or isinstance(v, float) for v in figsize_input)
            else:
                is_numeric_value = False
        except:
            is_numeric_value = False
            
        if is_numeric_value:
            figsize = figsize_input
        else:
            figsize = None

        return figsize


from .chart import Chart
import folium
from IPython.display import display

class Map(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Map object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._coordinate_column, 1)

        return is_promote
    
    def _check_requirements(self):
        """
        Check the requirements for generating tree visualization

        Returns:
            (list) popup_data: list of label name
        """
        popup_data = None
        if self._is_var_exist(self._coordinate_column, 1):
            new_data = self._add_point()
            if len(self._label_column) == 0:
                popup_data = new_data.coordinate_point
            else:
                popup_data = new_data[self._label_column[0]]
        else:
            popup_data = None
        
        return popup_data


    def plot(self):
        """
        Generate Image Grid visualization
        """
        if self._is_var_exist(self._coordinate_column, 1):
            self.draw_map()
        else:
            pass


    def draw_map(self):
        """
        Generate map visualization
        """
        popup_data = self._check_requirements()

        if popup_data is not None:
            data_point = self._add_point()
            #Initiate map folium object
            new_data = self.truncate_data(data_point)
            maps = folium.Map()

            #Marked the map folium object
            for i in range (len(new_data)):
                folium.Marker(
                    location=new_data.coordinate[i],
                    popup=popup_data[i]
                ).add_to(maps)

            display(maps)                

    def _add_point(self):
        """
        Add coordinate column for coordinate folium map

        Returns:
            (pandas.Dataframe): Dataframe with new coordinate column
        """
        copy_data = self.dataframe.copy()

        coor_var = self._coordinate_column[0]    
        #Get coordinate data (latitude and longitude)
        char_delete = 'Point()OINT'
        copy_data['coordinate_point'] = copy_data[coor_var]
        dataframe_new = copy_data.coordinate_point.astype(str).apply(lambda S:S.strip(char_delete))
        dataframe_new = dataframe_new.to_frame()
        new = dataframe_new[dataframe_new.columns[-1]].str.split(" ", n = 1, expand = True)
        new = new.astype('float64')
        copy_data['coordinate'] = new.apply(lambda x: list([x[1], x[0]]),axis=1)

        return copy_data

    def truncate_data(self, data):

        if len(data) > 2000 :
            truncate_data = data.head(2000)
            data = truncate_data
            print(f"Time limit exceed... Showing only 2000 coordinates")
        else:
            pass

        return data


from .chart import Chart
import seaborn as sns

class DensityPlot(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the DensityPlot object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._numerical_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating DensityPlot visualization

        Returns:
            (string) numerical_label: label of numerical column
            (string) label_name: label name
        """
        numerical_label = None
        label_name = None

        if self._is_var_exist(self._numerical_column, 1):
            numerical_label = self._numerical_column[0]
            self._item_var, self._categorical_column  = self._set_item_and_categorical()
            if len(self._categorical_column) > 0:
                label_name = self._categorical_column[0]

        return numerical_label, label_name      

    def filter_data(self):

        var_name = list(self.dataframe.columns)
        data = self.dataframe.copy()

        if len(self._date_column) > 0:
            filter_date_column = list(set(var_name) - set(self._date_column))
            data = data.filter(items=filter_date_column)
        else:
            pass

        return data

    def draw(self):

        numerical_label, label_name  = self._check_requirements()

        if label_name is not None:
            sns.displot(data=self.dataframe, x=numerical_label, hue=label_name, kind="kde")
            pass
        else:
            sns.displot(data=self.dataframe, x=numerical_label, kind="kde")
            pass
        







from .chart import Chart
import networkx as nx
import matplotlib.pyplot as plt

class Graph(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Graph object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
            kwargs (dictionary): Arbitrary keyword arguments.
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._uri_column, 2)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating graph visualization

        Returns:
            (list) filter_column: list of filter label name
            (bool) is_label_column: if column not uri column
        """
        filter_column = None
        is_label_column = False

        if self._is_var_exist(self._uri_column, 2):
            if len(self._uri_column) > len(self._label_column):
                filter_column = self._uri_column
            else:
                filter_column = self._sort_label_column(self._label_column)
                is_label_column = True
        
        return filter_column, is_label_column      

    def _sort_label_column(self, label_column):
        #sort based on unique value (ASC)
        unique_dict = {name:len(self.dataframe[name].unique()) for name in (label_column)}
        sort_dict = {k: v for k, v in sorted(unique_dict.items(), key=lambda item: item[1])}

        sorted_label_column = list(sort_dict.keys())

        return sorted_label_column

    def draw(self):
        """
        Generate graph visualization
        """
        is_label_column = True
        filter_column = None
        keyword_column = None
        filter_column, is_label_column = self._check_requirements()

        #check mode keyword
        self.mode_keyword = set_mode(self.kwargs.get('mode_keyword'))
        if self.mode_keyword is not None:
            keyword_column = self._check_variable_mode_keyword()
            

        #plot
        if filter_column is not None:
            self.figsize = set_figsize(self.kwargs.get('figsize'))
            #check if param figsize exist
            if self.figsize is not None:
                plt.figure(figsize=self.figsize)
            else:
                plt.figure(figsize=(20,15))
            try:
                #check if edge label exist
                if len(filter_column) > 2:
                    #check if label exist (not uri)
                    if keyword_column is not None:
                        graph, positions, edge_labels = self.create_graph_nx('source_node', 'target_node', 'edge_label')
                    elif is_label_column:
                        graph, positions, edge_labels = self.create_graph_nx(filter_column[0], filter_column[2], filter_column[1])
                    else:
                        graph, positions, edge_labels = self.create_graph_nx(filter_column[0], filter_column[2], filter_column[1])
                    nx.draw_networkx(graph, positions, arrowsize=15, node_color='#f0f8ff')
                    nx.draw_networkx_edge_labels(graph, pos=positions, edge_labels=edge_labels, font_color='r')
                else:
                    graph, positions, edge_labels = self.create_graph_nx(filter_column[0], filter_column[1])
                    nx.draw_networkx(graph, positions, arrowsize=15, node_color='#f0f8ff')
            finally:
                plt.show()

    def create_graph_nx(self, source_column, target_column, edge_column=None):
        """
        Create graph networkx

        Paramaters:
            (list) node_list: list of node
            (list) filter_column: list of parent and child name column

        Returns:
            (networkx.DiGraph) Graph: Digraph graph
        """
        Graph = nx.DiGraph()
        
        #add edges and edge_label to graph
        edge_label = {}
        for key, node in self.dataframe.iterrows():
            Graph.add_edges_from([(node[source_column],node[target_column])])
            if edge_column is not None:
                edge_label[(node[source_column],node[target_column])] = node[edge_column]


        #Getting positions for each node.
        positions = nx.kamada_kawai_layout(Graph)

        return Graph, positions, edge_label

    def _check_variable_mode_keyword(self):
        """
        Check the required var for generating graph visualization

        Returns:
            (list) filter_column: list of filter label name        
        """
        filter_column = None
        required_var = ['source_node', 'target_node', 'edge_label']
        exist_var = [name for name in self.dataframe.columns if name.startswith(tuple(required_var)) and self.dataframe[name].dtypes == 'string']
        miss_var = list(set(required_var)-set(exist_var))

        if len(miss_var) > 0:
            raise Exception(f"Missing required variable: {miss_var}")
        else:
            filter_column = required_var

        return filter_column

def set_mode(mode_input):
    """
    Setter of mode of mapping based on mode input

    Parameters:
        (bool) mode_input: The mode input

    Returns:
        (bool) mode: The result mode  
    """
    mode = None
    if mode_input is not None and isinstance(mode_input, bool) and mode_input == True:
        mode = mode_input
    else:
        mode = None
    
    return mode

def set_figsize(figsize_input):
    """
    Setter of figsize based on figsize input for matplotlib chart

    Parameters:
        (tuple) figsize_input: The figsize input

    Returns:
        (tuple) figsize: The result figsize  
    """
    figsize = None
    is_numeric_value = None

    try:
        if figsize_input is not None and len(figsize_input) == 2:
            is_numeric_value = all(isinstance(v, int) or isinstance(v, float) for v in figsize_input)
        else:
            is_numeric_value = False
    except:
        is_numeric_value = False
        
    if is_numeric_value:
        figsize = figsize_input
    else:
        figsize = None

    return figsize


import re
import pandas as pd
import importlib, inspect
from pandas import json_normalize
from SPARQLWrapper import SPARQLWrapper
from difflib import SequenceMatcher
import ssl

from .chartdict import chartdict as chart_dictionary

def set_chart(chart_input):
    """
    Setter of chart based on chart input

    :param (str) chart_input: The chart input

    :return: (str) chart: The available chart   
    """
    chart = chart_dictionary 
    charts = chart.keys()

    if chart_input is not None:
        lowercase_input = chart_input.lower()
        highest_prob = 0

        if lowercase_input in charts:
            chart = lowercase_input
        else:    
            for name in charts:
                prob_now = SequenceMatcher(None, lowercase_input, name).ratio()
                if prob_now > highest_prob and prob_now >= 0.5:
                    highest_prob = prob_now
                    chart = name
    else:
        chart = None

    return chart

def set_dataframe(sparql_query, sparql_endpoint, user, passwd):
    """
    Query the endpoint with the given query string and format the result table

    Parameters:
        (string) sparql_query: The sparql query.
        (string) sparql_endpoint: The sparql endpoint
        (string) user: The sparql endpoint basic authentication user
        (string) passwd: The sparql endpoint basic authentication password

    Returns:
        (pandas.Dataframe) result_table: The table of result    
    """

    sparql = SPARQLWrapper(sparql_endpoint)  

    sparql.setQuery(sparql_query)
    sparql.setReturnFormat('json')
    if user != None:
        ssl._create_default_https_context = ssl._create_unverified_context
        sparql.setCredentials(user, passwd)

    results = sparql.query().convert()
    table  = json_normalize(results["results"]["bindings"])

    data_table = table[[column_name for column_name in table.columns if column_name.endswith('.value')]]
    data_table.columns = data_table.columns.str.replace('.value$', '', regex=True)
    result_table = __convert_dtypes(data_table)
    
    return result_table

def __convert_dtypes(dataframe):
    """
    Convert data type each column of dataframe

    Parameters:
        (pandas.Dataframe) dataframe: The table

    Returns:
        (pandas.Dataframe) table: The result table             
    """

    for column in dataframe:
        try:
            dataframe[column] = dataframe[column].astype('string')
        except ValueError:
            pass

    for column in dataframe:
        try:
            dataframe[column] = dataframe[column].astype('datetime64')
        except ValueError:
            pass

    for column in dataframe:
        try:
            dataframe[column] = dataframe[column].astype('float64')
        except (ValueError, TypeError):
            pass

    return dataframe

def generate_charts_dictionary():
    """
        Get dictionary of chart type

        Returns:
            (dict) chartdict: dictionary of visualization chart type
    """
    keys = []
    values = []
    for name, mod in inspect.getmembers(importlib.import_module("VizKG.charts"), inspect.ismodule):
            keys.append(name)

    for name, cls in inspect.getmembers(importlib.import_module("VizKG.charts"), inspect.isclass):
            values.append(cls)

    chartdict = {keys[i]: values[i] for i in range(len(values))}
    chartdict.pop("chart")

    return chartdict    


import sys
import random
from .utils import set_chart, set_dataframe, chartdict
from .charts import Chart
class VizKG:
  """
  Instantiate VizKG object.
  
  Attributes:
      sparql_query (string): The SPARQL query to retrieve.
      sparql_service_url (string): The SPARQL endpoint URL.
      chart (string): Type of visualization
        Options = {'Table', 'ImageGrid', 'Timeline' 'Graph' 
                  'Map', 'Tree','WordCloud', 'Dimensions',
                  'LineChart', 'BarChart', 'Histogram',
                  'DensityPlot', 'TreeMap' ,'SunBurstChart', 
                  'HeatMap' ,'PieChart', 'DonutChart',
                  'BoxPlot' ,'ViolinPlot', 'AreaChart',
                  'StackedAreaChart', 'ScatterChart', 
                  'BubbleChart', 'RadarChart'}.
      **figsize (float, float): Width, height in inches of matplotlib plot 
  """

  def __init__(self, sparql_query, sparql_service_url, user=None, passwd=None, chart=None, **kwargs):
      """
      Constructs all the necessary attributes for the vizKG object

      Parameters:
          sparql_query (string): The SPARQL query to retrieve.
          sparql_service_url (string): The SPARQL endpoint URL.
          user (string): The sparql endpoint basic authentication user
          passwd (string): The sparql endpoint basic authentication password
          chart (string): Type of visualization
      """

      self.sparql_query = sparql_query
      self.sparql_service_url = sparql_service_url
      self.user = user
      self.passwd = passwd
      self.chart = set_chart(chart)
      self.kwargs = kwargs

      self.__data = set_dataframe(sparql_query, sparql_service_url, user, passwd)
      self.__candidate_visualization = self.__find_candidate()
      self.dataframe = self.__data
      self.candidate_visualization = self.__candidate_visualization     

  def plot(self):
      """
      Plot visualization with suitable corresponding chart

      """
      chart_list = chartdict.keys()
      figure = None
      if len(self.__data) != 0:
        if self.chart not in chart_list:
          if len(self.__candidate_visualization) > 1:
            print(f"You haven’t selected the chart type for your query result visualization.")
            print(f"Based on your query result data, we suggest to choose one of the following chart type: {self.__candidate_visualization}\n")
            self.__plot_randomize(self.__candidate_visualization)
          else:
            figure = chartdict["table"](self.__data, self.kwargs)
            figure.plot()      
        else:
          if self.chart in self.__candidate_visualization:
            figure = chartdict[self.chart](self.__data, self.kwargs)
            figure.plot()
          else:
            print(f"Based on your query result data, we suggest to choose one of the following chart type: {self.__candidate_visualization}\n")
      else:
        print("No matching records found")

  def __find_candidate(self):
      """
      Find candidate of visualization

      Returns:
          (list) candidate: List of recommendation chart name      
      """
      chart_list = list(chartdict.keys())
      candidate = []
      for idx,name in enumerate(chart_list):
          check = chartdict[name.lower()](self.__data, self.kwargs)
          if check.promote_to_candidate():
            candidate.append(name)
      return candidate

  def __plot_randomize(self, candidate_visualization):
      """
      Plot two of recommendation chart chart

      Returns:
          (list) candidate: List of recommendation chart name      
      """
      list_of_random_items = random.sample(candidate_visualization, 2)
      print(f"We show below two of them {tuple(list_of_random_items)} as illustrations: ")
      for idx,name in enumerate(list_of_random_items):
        figure = chartdict[name.lower()](self.__data, self.kwargs)
        figure.plot()

sys.modules[__name__] = VizKG


from .chart import Chart
import seaborn as sns
import plotly.express as px

class BarChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the BarChart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        item_column, categorical_column = self._set_item_and_categorical()
        is_promote = self._is_var_exist(self._numerical_column, 1) and self._is_var_exist(item_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating BarChart visualization

        Returns:
            (string) int_label: numerical label 
            (list) item_column: item_column
            (list) categorical_column: categorical_column
        """
        int_label = None
        item_column = None
        categorical_column = None

        if self._is_var_exist(self._numerical_column, 1):
            int_label = self._numerical_column[0]
            if self._is_var_exist(self._label_column, 1):
                item_column, categorical_column = self._set_item_and_categorical()
        
        return int_label, item_column, categorical_column    

    def draw(self):
        """
        Generate BarChart visualization
        """
        numerical_label, item_column, categorical_column  = self._check_requirements()
        
        #check orientation
        orientation = None
        if len(categorical_column) > 0:
            orientation = self._check_orientation(item_column[0],categorical_column[0])
        else:
            orientation = self._check_orientation(item_column[0])

        if len(categorical_column) > 0:
            if orientation is not None:
                fig = px.bar(self.dataframe, x=numerical_label, y=item_column[0], color=categorical_column[0])
                fig.show()
            else:
                fig = px.bar(self.dataframe, x=item_column[0], y=numerical_label, color=categorical_column[0])
                fig.show()
        else:
            if orientation is not None:
                data = self.dataframe.sort_values(by=[numerical_label])
                fig = px.bar(data, x=numerical_label, y=item_column[0])
                fig.show()
            else:
                data = self.dataframe.sort_values(by=[numerical_label], ascending=False)
                fig = px.bar(data, x=item_column[0], y=numerical_label)
                fig.show()             


    def _check_orientation(self, axis_label, group_label=None, max_number=6):
        """
        Check the requirements for changing orientation, returns None if horizontal

        Returns:
            (string) orientation: label for axis
        """
        orientation = None
        num_box = 0
        num_axis = len(self.dataframe[axis_label].unique())
        num_box = 0

        if group_label is not None:
            num_group = len(self.dataframe[group_label].unique())
            num_box = num_axis + num_group
        else:
            num_box = num_axis

        if num_box > max_number:
            orientation = 'Horizontal'

        return orientation


from .chart import Chart
import pandas as pd
import matplotlib.pyplot as plt
from wordcloud import STOPWORDS, WordCloud as wrdcld

class WordCloud(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the WordCloud object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._label_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _word_result(self):
        """
        Compile dataframe to one variable

        Parameters:
            (pandas.Dataframe) dataframe: The dataframe

        Returns:
            (string) words: The word result
        """
        #Merge into one column
        new_data = self.dataframe[self._label_column]
        new_data_flat = list(pd.Series(new_data.values.ravel('F')))

        #Merge into one variable
        words = " ".join([str(element) for element in new_data_flat])

        return words

    def draw(self):
        """
        Display WordCloud visualizations

        Parameters:
            (string) words: the visualized words
        """
        if self._is_var_exist(self._label_column, 1):
            #initiate words
            words = self._word_result()
            #initiate wordcloud object
            stopwords = set(STOPWORDS) 
            wordcloud = wrdcld(
                            width = 800, height = 800, 
                            background_color ='white', 
                            stopwords = stopwords, 
                            min_font_size = 10
                            ).generate(words) 
            
            # plot the WordCloud image
            self.figsize = self.__set_figsize(self.kwargs.get('figsize'))
            #check if param figsize exist
            if self.figsize is not None:
                plt.figure(figsize = self.figsize, facecolor = None) 
                plt.imshow(wordcloud) 
                plt.axis("off") 
                plt.tight_layout(pad = 0)
            else:                 
                plt.figure(figsize = (8, 8), facecolor = None) 
                plt.imshow(wordcloud) 
                plt.axis("off") 
                plt.tight_layout(pad = 0)

    @staticmethod
    def __set_figsize(figsize_input):
        """
        Setter of figsize based on figsize input for matplotlib chart

        Parameters:
            (tuple) figsize_input: The figsize input

        Returns:
            (tuple) figsize: The result figsize  
        """
        figsize = None
        is_numeric_value = None

        try:
            if figsize_input is not None and len(figsize_input) == 2:
                is_numeric_value = all(isinstance(v, int) or isinstance(v, float) for v in figsize_input)
            else:
                is_numeric_value = False
        except:
            is_numeric_value = False
            
        if is_numeric_value:
            figsize = figsize_input
        else:
            figsize = None

        return figsize


import re
import statistics

class Chart():
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Chart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
            kwargs: Arbitrary keyword arguments.
        """
        self.dataframe = dataframe
        self.kwargs = kwargs
                
        self._uri_column = self._set_uri_column()
        self._date_column = self._set_date_column()
        self._numerical_column = self._set_numerical_column()
        self._coordinate_column = self._set_coordinate_column()
        self._img_column = self._set_image_column()
        self._label_column = self._set_label_column()
        
    def promote_to_candidate(self):
        "Check required variable to generate chart"
        pass

    def plot(self):
        "Generate visualization"
        pass

    def _is_var_exist(self, column, request=1):
        """
        Check if list exist return True if exist

        Parameters:
            (int) request:number of required column
        
        Returns:
            (boolena) is_exist: True if list exist
        """
        is_exist = False
        if len(column) >= request:
            is_exist = True
        else:
            is_exist = False
        
        return is_exist

    def _set_label_column(self):
        """
        Get label column name of dataframe based on 'string' dtypes 
            with excluded uri, image url and coordinate column

        :return: (list) label_column: list of label column        
        """
        str_column = list(self.dataframe.columns)
        
        #exclude uri, image url, coordinate column
        excluded_column = self._uri_column + self._img_column + self._coordinate_column + self._numerical_column + self._date_column
        label_column = [i for i in str_column + excluded_column if i not in str_column or i not in excluded_column]

        return label_column

    def _set_item_and_categorical(self):
        """
        Set item and categorical var from label column
        set categorical var if unique value <= (len(self.dataframe) / 2)

        :return: (list,list) list_item_col, list_of_categorical_variable: list of name        
        """
        item_col = []
        categorical_col = []

        filter_col = []
        if len(self._label_column) > 0:
            filter_col = self._label_column

        unique_dict = {name:len(self.dataframe[name].unique()) for name in (filter_col)}
        sort_dict = {k: v for k, v in sorted(unique_dict.items(), key=lambda item: item[1])}
        for name, value in sort_dict.items():
            if value <= (len(self.dataframe) / 2):
                categorical_col.append(name)
            else:
                item_col.append(name)
        return item_col, categorical_col

    def _set_date_column(self):
        """
        Get date column name of dataframe based on date data type
        """
        date_column = [name for name in self.dataframe.columns if self.dataframe[name].dtypes == 'datetime64[ns]']

        return date_column

    def _set_numerical_column(self):
        """
        Get date column name of dataframe based on date data type
        """
        numerical_column = [name for name in self.dataframe.columns if self.dataframe[name].dtypes == 'float64']

        return numerical_column 

    def _set_uri_column(self):
        """
        Get date column name of dataframe based on date data type
        """
        #Regex pattern
        """
        Get uri column name of dataframe based on regex pattern

        :return: (list) uri_column: list of uri variable
        """
        #Regex pattern
        pattern_url = r"^(?:http(s)?:\/\/)[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:/?#[\]@!\$&'\(\)\*\+,;=.]+$(?<!.[jpg|gif|png|JPG|PNG])" 
        uri_column = self.set_column_based_on_regex(pattern_url)

        return uri_column

    def _set_image_column(self):
        """
        Get image column name of dataframe based on regex pattern

        :return: (list) image_column: list of image variable
        """
        #Regex pattern
        pattern_img = r"^http(s)?://(?:[a-z0-9\-]+\.)+[a-z]{2,6}(?:/[^/#?]+)+\.(?:jpg|jpeg|gif|png|JPG|JPEG|Jpeg)$"        
        image_column = self.set_column_based_on_regex(pattern_img)

        return image_column

    def _set_coordinate_column(self):
        """
        Get coordinate column name of dataframe based on regex pattern

        :return: (list) coordinate_column: list of coordinate variable
        """
        #Regex pattern
        pattern_coordinate1 = r"^Point"
        pattern_coordinate2 = r"^POINT"
        coordinate_column1 = self.set_column_based_on_regex(pattern_coordinate1)
        coordinate_column2 = self.set_column_based_on_regex(pattern_coordinate2)
        
        coordinate_column = coordinate_column1 + coordinate_column2
        return coordinate_column

    def set_column_based_on_regex(self, pattern):
        """
        Set list of column name based on regex matching

        :return: (list) column: list of name
        """
        list_column = []

        for i in range (len(self.dataframe.columns)):
            column_name = self.dataframe.columns[i]
            column = self.dataframe[self.dataframe.columns[i]]
            is_matched_column = self.check_data_per_column(column, pattern)
            if is_matched_column:
                list_column.append(column_name)
        
        return list_column

    def check_data_per_column(self, column, pattern):
        """
        Check entire data per column of dataframe if matched with regex pattern

        Parameters:
            (pandas.Dataframe) column: column of dataframe
            (string) pattern: regex pattern

        Returns:
            (boolen) boolean_check: The result table             
        """
        boolean_check = False
        for datapoint in range(len(column)):
            data = column.iloc[datapoint]
            try:
                if re.match(pattern, data):
                    boolean_check = True
            except TypeError:
                pass
                
        return boolean_check




from .chart import Chart
import plotly.graph_objects as go


class RadarChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Radar Chart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._label_column, 1) and self._is_var_exist(self._numerical_column, 3)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating Radar Chart visualization

        Returns:
            (string) label_name: label name
            (list) numerical_column: list of numerical column
        """
        label_name = None
        numerical_column = None
        
        if self._is_var_exist(self._numerical_column, 3):
            numerical_column = self._numerical_column
            if self._is_var_exist(self._label_column, 1):
                label_name = self._label_column[0]
        
        return label_name, numerical_column    

    def draw(self):
        """
        Generate Radar Chart visualization
        """
        label_name, numerical_column  = self._check_requirements()

        if label_name is not None and numerical_column is not None:
            categories = numerical_column
            data_label = self.dataframe[label_name]
            data_numeric = self.dataframe[numerical_column]

            list_number = []

            fig = go.Figure()

            for i in range (len(data_numeric)):
                idx_data_numeric = (list(data_numeric.iloc[i]))
                fig.add_trace(go.Scatterpolar(
                    r=idx_data_numeric,
                    theta=categories,
                    fill='toself',
                    name=data_label[i]
                ))
                list_number.append(idx_data_numeric)

            fig.update_layout(
                polar=dict(
                    radialaxis=dict(
                    visible=True,
                    range=[0, max(list_number)]
                    )),
                showlegend=False
            )

            fig.show()






from .chart import Chart
import plotly.express as px

class ViolinPlot(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the violinPlot object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._numerical_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating violinPlot visualization

        Returns:
            (string) numerical_label: label of numerical column
            (list) group_column: categorical column
        """
        numerical_label = None
        group_column = None
        item_col, categorical_col = self._set_item_and_categorical()

        if self._is_var_exist(self._numerical_column, 1):
            numerical_label = self._numerical_column[0]
            if self._is_var_exist(categorical_col, 1):
                group_column= categorical_col
                
        return numerical_label, group_column    

    def draw(self):
        """
        Generate violinPlot visualization
        """
        numerical_label, group_column  = self._check_requirements()

        if numerical_label is not None and group_column is not None:
            if len(group_column) > 1:
                fig = px.violin(self.dataframe, x=group_column[1], y=numerical_label, color=group_column[0])
                fig.show()
            else:
                fig = px.violin(self.dataframe, x=group_column[0], y=numerical_label)
                fig.show()
        elif numerical_label is not None:
            fig = px.violin(self.dataframe, y=numerical_label)
            fig.show()
        else:
            pass    


from .chart import Chart
import plotly.graph_objects as go

class Dimensions(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Dimensions object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._label_column, 2)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating dimension visualization

        Returns:
            (list) dataframe_to_list: list of dataframe
        """
        dataframe_to_list = None
        if self._is_var_exist(self._label_column, 2):
            dataframe_to_list = []
            for column in self._label_column:
                dataframe_to_list += self.dataframe[column].tolist()
        
        return dataframe_to_list    

    def draw(self):
        """
        Generate Dimensions visualization
        """
        dataframe_to_list = self._check_requirements()

        if dataframe_to_list is not None:
            #plot
            figure = go.Figure(data=[go.Sankey(
                node = dict(
                    label = dataframe_to_list,
                ),
                link = dict(
                    source = self._index_data(dataframe_to_list), # indices correspond to labels, eg A1, A2, A1, B1, ...
                    target = self._index_data(dataframe_to_list, type_link='target'),
                    value = [1 for i in range(len(dataframe_to_list)-self.dataframe.shape[0])]
                ))])

            figure.show()

    def _index_data(self, dataframe_to_list, type_link='source'):
        """
        Return indices correspond to type_link labels

        Parameters:
            (string) type_link: Type of link {'source' or target}
                                DEFAULT: 'source'
        Returns:
            (list) indices: index list of Type of link                        
        """
        curr_key = 0
        indices = [0]
        curr_value = dataframe_to_list[0]
        first_row = [dataframe_to_list[0]] 
        data = dataframe_to_list[:-self.dataframe.shape[0]]

        if type_link == 'target':
          curr_value = dataframe_to_list[self.dataframe.shape[0]]
          first_row = [dataframe_to_list[self.dataframe.shape[0]]]
          data = dataframe_to_list[self.dataframe.shape[0]:]

        for key,value in enumerate(data):
          if value != curr_value :
            if value in first_row:
              curr_key = first_row.index(value)
              curr_value = value
              indices.append(curr_key)
              first_row.append(curr_value)
            else:
              indices.append(key)
              first_row.append(value)
              curr_value = value
              curr_key = key
          elif value == curr_value:
            if key != 0:
              indices.append(curr_key)
              first_row.append(curr_value)
              
        if type_link == 'target':
          indices = [i+self.dataframe.shape[0] for i in indices]

        return indices


from .chart import Chart
import plotly.express as px

class AreaChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the AreaChart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._numerical_column, 1) and self._is_var_exist(self._date_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating AreaChart visualization

        Returns:
            (string) date_label: date label  for axis-x
            (string) int_label: numerical label for axis-y
            (string) label_name: label name
        """
        date_label = None
        int_label = None
        label_name = None

        if self._is_var_exist(self._date_column, 1):
            date_label = self._date_column[0]
            if self._is_var_exist(self._numerical_column, 1):
                int_label = self._numerical_column[0]
                if self._is_var_exist(self._label_column, 1):
                    label_name = self._label_column[0]
        
        return date_label, int_label, label_name          

    def draw(self):
        """
        Generate AreaChart visualization
        """
        date_label, numerical_label, label_name  = self._check_requirements()

        if label_name is not None:
            #plot
            fig = px.area(self.dataframe, x=date_label, y=numerical_label, color=label_name, line_group=label_name)
            fig.show()
        else:
            fig = px.area(self.dataframe, x=date_label, y=numerical_label)
            fig.show()




from .chart import Chart
import plotly.express as px

class ScatterChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the ScatterChart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._numerical_column, 2)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating ScatterChart visualization

        Returns:
            (list) numerical_columns: list of numerical column
            (string) label_name: label name
        """
        numerical_columns = None
        label_name = None

        if self._is_var_exist(self._numerical_column, 2):
            numerical_columns = self._numerical_column
            if len(self._label_column) > 0:
                label_name = self._label_column[0]
        
        return numerical_columns, label_name    

    def draw(self):
        """
        Generate ScatterChart visualization
        """
        numerical_columns, label_name = self._check_requirements()

        if numerical_columns is not None:
            x_label = numerical_columns[0]
            y_label = numerical_columns[1]
            if label_name is not None:
                fig = px.scatter(self.dataframe, x=x_label, y=y_label, color=label_name)
                fig.show()
            else:
                fig = px.scatter(self.dataframe, x=x_label, y=y_label)
                fig.show()                


from .chart import Chart
from anytree import Node, RenderTree

class Tree(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Tree object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._uri_column, 2)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating tree visualization

        Returns:
            (list) filter_column: list of filter label name
        """
        filter_column = None
        if self._is_var_exist(self._uri_column, 2):
            if (len(self._label_column)) == (len(self._uri_column)):
                filter_column = self._label_column
            else:
                filter_column = self._uri_column
        else:
            pass            
        
        return filter_column
    
    def draw(self):
        """
        Generate tree visualization
        """
        #filter_column
        filter_column = self._check_requirements()

        if filter_column is not None:
            #Extract selected column as new dataframe
            data = self.dataframe[filter_column].copy()

            for i in range (len(filter_column)):
                nodes = {}
                if i == len(filter_column) - 1:
                    break
                for parent, child in zip(data.iloc[:, i],data.iloc[:, i+1]):
                    self.add_nodes(nodes, parent, child)                
            
                roots = list(data[~data.iloc[:, i].isin(data.iloc[:, i+1])][data.columns[i]].unique())
                for root in roots:         # you can skip this for roots[0], if there is no forest and just 1 tree
                    for pre, _, node in RenderTree(nodes[root]):
                        print("%s%s" % (pre, node.name))

    @staticmethod    
    def add_nodes(nodes, parent, child):
        """
        Set parent nodes with corresponding child nodes
        """
        if parent not in nodes:
            nodes[parent] = Node(parent)  
        if child not in nodes:
            nodes[child] = Node(child)
            nodes[child].parent = nodes[parent]


from .chart import Chart
import numpy as np
import random 
import matplotlib.pyplot as plt

class BubbleChart(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the BubbleChart object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        item_col, categorical_col = self._set_item_and_categorical()
        is_promote = self._is_var_exist(item_col, 1) and self._is_var_exist(self._numerical_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating BubbleChart visualization

        Returns:
            (string) numerical_label: label column with float data type
            (string) label_name: label column with string data type
        """
        numerical_label = None
        label_name = None
        if self._is_var_exist(self._numerical_column, 1):
            numerical_label = self._numerical_column[0]
            if len(self._label_column) > 0:
                label_name = self._label_column[-1]
        
        return numerical_label, label_name

    def draw(self):
        """
        Generate table visualization
        """
        numerical_label, label_name = self._check_requirements()

        if numerical_label is not None and label_name is not None:
            self.figsize = set_figsize(self.kwargs.get('figsize'))
            bubble_chart = DrawBubbleChart(area=self.dataframe[numerical_label], bubble_spacing=2)
            bubble_chart.draw(self.dataframe[label_name], self.figsize)


class DrawBubbleChart:
    def __init__(self, area, bubble_spacing=0):
        """
        Setup for bubble collapse.

        Parameters
        ----------
        area : array-like
            Area of the bubbles.
        bubble_spacing : float, default: 0
            Minimal spacing between bubbles after collapsing.

        Notes
        -----
        If "area" is sorted, the results might look weird.
        """
        area = np.asarray(area)
        r = np.sqrt(area / np.pi)

        self.bubble_spacing = bubble_spacing
        self.bubbles = np.ones((len(area), 4))
        self.bubbles[:, 2] = r
        self.bubbles[:, 3] = area
        self.maxstep = 2 * self.bubbles[:, 2].max() + self.bubble_spacing
        self.step_dist = self.maxstep / 2

        # calculate initial grid layout for bubbles
        length = np.ceil(np.sqrt(len(self.bubbles)))
        grid = np.arange(length) * self.maxstep
        gx, gy = np.meshgrid(grid, grid)
        self.bubbles[:, 0] = gx.flatten()[:len(self.bubbles)]
        self.bubbles[:, 1] = gy.flatten()[:len(self.bubbles)]

        self.com = self.center_of_mass()
        
        # initiate color of bubbles
        group_of_items = ['#5A69AF', '#579E65', '#F9C784', '#FC944A', '#F24C00',
                          '#00B825', '#e6194b', '#3cb44b', '#ffe119', '#4363d8', 
                          '#f58231', '#911eb4', '#46f0f0', '#f032e6', '#bcf60c',
                          '#fabebe', '#008080', '#e6beff', '#9a6324', '#fffac8',
                          '#800000', '#aaffc3', '#808000', '#ffd8b1', '#000075',
                          '#808080']
        num_to_select = len(self.bubbles)
        self.colors = [random.choice(group_of_items) for _ in range(num_to_select)]

    def center_of_mass(self):
        return np.average(
            self.bubbles[:, :2], axis=0, weights=self.bubbles[:, 3]
        )

    def center_distance(self, bubble, bubbles):
        return np.hypot(bubble[0] - bubbles[:, 0],
                        bubble[1] - bubbles[:, 1])

    def outline_distance(self, bubble, bubbles):
        center_distance = self.center_distance(bubble, bubbles)
        return center_distance - bubble[2] - \
            bubbles[:, 2] - self.bubble_spacing

    def check_collisions(self, bubble, bubbles):
        distance = self.outline_distance(bubble, bubbles)
        return len(distance[distance < 0])

    def collides_with(self, bubble, bubbles):
        distance = self.outline_distance(bubble, bubbles)
        idx_min = np.argmin(distance)
        return idx_min if type(idx_min) == np.ndarray else [idx_min]

    def collapse(self, n_iterations=50):
        """
        Move bubbles to the center of mass.

        Parameters
        ----------
        n_iterations : int, default: 50
            Number of moves to perform.
        """
        for _i in range(n_iterations):
            moves = 0
            for i in range(len(self.bubbles)):
                rest_bub = np.delete(self.bubbles, i, 0)
                # try to move directly towards the center of mass
                # direction vector from bubble to the center of mass
                dir_vec = self.com - self.bubbles[i, :2]

                # shorten direction vector to have length of 1
                dir_vec = dir_vec / np.sqrt(dir_vec.dot(dir_vec))

                # calculate new bubble position
                new_point = self.bubbles[i, :2] + dir_vec * self.step_dist
                new_bubble = np.append(new_point, self.bubbles[i, 2:4])

                # check whether new bubble collides with other bubbles
                if not self.check_collisions(new_bubble, rest_bub):
                    self.bubbles[i, :] = new_bubble
                    self.com = self.center_of_mass()
                    moves += 1
                else:
                    # try to move around a bubble that you collide with
                    # find colliding bubble
                    for colliding in self.collides_with(new_bubble, rest_bub):
                        # calculate direction vector
                        dir_vec = rest_bub[colliding, :2] - self.bubbles[i, :2]
                        dir_vec = dir_vec / np.sqrt(dir_vec.dot(dir_vec))
                        # calculate orthogonal vector
                        orth = np.array([dir_vec[1], -dir_vec[0]])
                        # test which direction to go
                        new_point1 = (self.bubbles[i, :2] + orth *
                                      self.step_dist)
                        new_point2 = (self.bubbles[i, :2] - orth *
                                      self.step_dist)
                        dist1 = self.center_distance(
                            self.com, np.array([new_point1]))
                        dist2 = self.center_distance(
                            self.com, np.array([new_point2]))
                        new_point = new_point1 if dist1 < dist2 else new_point2
                        new_bubble = np.append(new_point, self.bubbles[i, 2:4])
                        if not self.check_collisions(new_bubble, rest_bub):
                            self.bubbles[i, :] = new_bubble
                            self.com = self.center_of_mass()

            if moves / len(self.bubbles) < 0.1:
                self.step_dist = self.step_dist / 2

    def draw(self, labels, figsize_input=None):
        """
        Draw the bubble plot.

        Parameters
        ----------
        ax : matplotlib.axes.Axes
        labels : list
            Labels of the bubbles.
        colors : list
            Colors of the bubbles.
        """
        self.collapse()

        #check if param figsize exist
        if figsize_input is not None:
            fig, ax = plt.subplots(figsize=figsize_input, subplot_kw=dict(aspect="equal"))
            for i in range(len(self.bubbles)):
                circ = plt.Circle(
                    self.bubbles[i, :2], self.bubbles[i, 2], 
                    color=self.colors[i],
                    )
                ax.add_patch(circ)
                ax.text(*self.bubbles[i, :2], labels[i],
                        horizontalalignment='center', verticalalignment='center')

            ax.axis("off")
            ax.relim()
            ax.autoscale_view()
            plt.show()
        else:
            fig, ax = plt.subplots(figsize=(10,8), subplot_kw=dict(aspect="equal"))
            for i in range(len(self.bubbles)):
                circ = plt.Circle(
                    self.bubbles[i, :2], self.bubbles[i, 2], 
                    color=self.colors[i],
                    )
                ax.add_patch(circ)
                ax.text(*self.bubbles[i, :2], labels[i],
                        horizontalalignment='center', verticalalignment='center')

            ax.axis("off")
            ax.relim()
            ax.autoscale_view()
            plt.show()


def set_figsize(figsize_input):
    """
    Setter of figsize based on figsize input for matplotlib chart

    Parameters:
        (tuple) figsize_input: The figsize input

    Returns:
        (tuple) figsize: The result figsize  
    """
    figsize = None
    is_numeric_value = None

    try:
        if figsize_input is not None and len(figsize_input) == 2:
            is_numeric_value = all(isinstance(v, int) or isinstance(v, float) for v in figsize_input)
        else:
            is_numeric_value = False
    except:
        is_numeric_value = False
        
    if is_numeric_value:
        figsize = figsize_input
    else:
        figsize = None

    return figsize


from .chart import Chart
import plotly.express as px

class TreeMap(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the TreeMap object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._label_column, 1)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for generating TreeMap visualization

        Returns:
            (list) label_column: label name
            (list) numerical_var: numerical variable
        """
        label_column = None
        numerical_var = None
        
        if self._is_var_exist(self._label_column, 1):
            label_column = self._label_column
            if self._is_var_exist(self._numerical_column):
                numerical_var = self._numerical_column[0]

        
        return label_column, numerical_var    

    def draw(self):
        """
        Generate TreeMap visualization
        """
        label_column, numerical_var  = self._check_requirements()

        if numerical_var is not None:
            fig = px.treemap(self.dataframe, values=numerical_var, path=label_column)
            fig.show()
        else:
            fig = px.treemap(self.dataframe, path=label_column)
            fig.show()                                



import codecs
import os
import re
from setuptools import find_packages, setup

def local_file(file):
  return codecs.open(
    os.path.join(os.path.dirname(__file__), file), 'r', 'utf-8'
)

install_reqs = [
  line.strip()
  for line in local_file('requirements.txt').readlines()
  if line.strip() != ''
]

# Get the long description from the README file
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='VizKG',
    packages=['VizKG', 'VizKG.charts', 'VizKG.utils'],
    version='1.0.9',
    description='Visualization library for SPARQL query results',
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls = {
    "Source Code": "https://github.com/fadirra/vizkg",
    "Demo" : "https://www.youtube.com/watch?v=i0dd_-PRxlI"
    },
    author='Hana',
    install_requires=install_reqs,
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.7'
)


from .chart import Chart
import plotly.figure_factory as ff
from IPython.display import display
import pandas as pd

class Table(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Table object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = len(self.dataframe) > 0

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def draw(self):
        """
        Generate table visualization
        """
        if len(self.dataframe) > 1000 :
            fig = ff.create_table(self.dataframe)
            fig.show()
        else:
            with pd.option_context('display.max_rows', None, 'display.max_columns', None):
                display(self.dataframe)    


from .chart import Chart
import plotly.express as px
import datetime

class Timeline(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the Timeline object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._date_column, 1) and (self._is_var_exist(self._label_column, 1) or self._is_var_exist(self._uri_column, 1))

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def _check_requirements(self):
        """
        Check the requirements for Timeline visualization

        Returns:
            (list) date_column: label for axis-x
            (list) label_name: label for axis-y
        """
        date_column = None
        label_name = None

        if self._is_var_exist(self._date_column, 1):
            date_column = self._date_column
            if len(self._label_column) == 0:
                if len(self._uri_column) > 0:
                    label_name = self._uri_column[0]
                else:
                    label_name = None
            else:
                label_name = self._label_column[0]
        
        return date_column, label_name


    def draw(self):
        """
        Generate Timeline visualization
        """
        date_column, label_name = self._check_requirements()

        if date_column is not None and label_name is not None:
            if len(date_column) >= 2:
                if self.dataframe[date_column[0]][0] > self.dataframe[date_column[1]][0]:
                    date_column[1],date_column[0] = date_column[0],date_column[1]
                fig = px.timeline(self.dataframe, x_start=date_column[0], x_end=date_column[1], 
                                y=label_name, color=label_name)
                fig.update_yaxes(autorange="reversed")
                fig.show()
            else:
                data = self.dataframe.sort_values(by=[date_column[0]])
                range_time = data[date_column[0]][0] - data[date_column[0]][len(self.dataframe)-1]
                add_column = self.dataframe.copy()

                if range_time <= datetime.timedelta(days=30):
                    add_column['T+1'] = [add_column[date_column[0]][i] + datetime.timedelta(days=1) for i in range (len(add_column))]
                elif range_time > datetime.timedelta(days=30) and range_time <= datetime.timedelta(days=365):
                    add_column['T+1'] = [add_column[date_column[0]][i] + datetime.timedelta(days=15) for i in range (len(add_column))]
                else:
                    add_column['T+1'] = [add_column[date_column[0]][i] + datetime.timedelta(days=365) for i in range (len(add_column))]

                fig = px.timeline(add_column, x_start=date_column[0], x_end='T+1', 
                                    y=label_name, color=label_name, hover_data={'T+1':False})
                fig.update_yaxes(autorange="reversed")
                fig.show()



from .chart import Chart
import matplotlib.pyplot as plt
import seaborn as sns

class HeatMap(Chart):
    def __init__(self, dataframe, kwargs):
        """
        Constructs all the necessary attributes for the HeatMap object

        Parameters:
            dataframe (pandas.Dataframe): The dataframe
        """
        Chart.__init__(self, dataframe, kwargs)

    def promote_to_candidate(self):

        is_promote = self._is_var_exist(self._numerical_column, 2)

        return is_promote

    def plot(self):
        """
        Generate visualization
        """
        if self.promote_to_candidate():
            self.draw()
        else:
            pass

    def draw(self):
        """
        Generate HeatMap visualization
        """

        if self._is_var_exist(self._numerical_column, 2):

            self.figsize = self.__set_figsize(self.kwargs.get('figsize'))
            #check if param figsize exist
            if self.figsize is not None:
                plt.figure(figsize=self.figsize)
                sns.heatmap(self.dataframe.corr(), annot = True)
                plt.show(block=True)
            else:                 
                #plot HeatMap
                plt.figure(figsize=(13,8))
                sns.heatmap(self.dataframe.corr(), annot = True)
                plt.show(block=True)

    @staticmethod
    def __set_figsize(figsize_input):
        """
        Setter of figsize based on figsize input for matplotlib chart

        Parameters:
            (tuple) figsize_input: The figsize input

        Returns:
            (tuple) figsize: The result figsize  
        """
        figsize = None
        is_numeric_value = None

        try:
            if figsize_input is not None and len(figsize_input) == 2:
                is_numeric_value = all(isinstance(v, int) or isinstance(v, float) for v in figsize_input)
            else:
                is_numeric_value = False
        except:
            is_numeric_value = False
            
        if is_numeric_value:
            figsize = figsize_input
        else:
            figsize = None

        return figsize